use crate::{schema, UIDCError};
use microrm::{prelude::*, schema::entity::EntityID};

#[derive(Debug)]
pub struct ScopeInterface;

#[derive(Debug, clap::Subcommand)]
pub enum ScopeCommands {
    Create { name: String },
}

impl microrm::cli::EntityInterface for ScopeInterface {
    type Error = UIDCError;
    type Entity = schema::Scope;
    type Context = microrm::schema::Stored<schema::Realm>;
    type CustomCommand = ScopeCommands;

    fn run_custom(
        ctx: &Self::Context,
        cmd: Self::CustomCommand,
        query_ctx: impl Queryable<EntityOutput = Self::Entity> + Insertable<Self::Entity>,
    ) -> Result<(), Self::Error> {
        match cmd {
            ScopeCommands::Create { name } => {
                query_ctx.insert(schema::Scope {
                    realm: ctx.id(),
                    shortname: name,
                    roles: Default::default(),
                })?;
            }
        }

        Ok(())
    }

    fn should_override(
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> bool {
        field == "realm"
    }

    fn override_for(
        ctx: &Self::Context,
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> String {
        if field == "realm" {
            format!("{}", ctx.id().into_raw())
        } else {
            unreachable!()
        }
    }
}
